<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Controller\Adminhtml\Configuration;

use Exception;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Montonio\Shipping\Model\Shipping\ShippingsConfigProvider;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use Montonio\Shipping\Gateway\Shippings\MontonioSDK;
use Montonio\Shipping\Model\ResourceModel\ShippingPoints\CollectionFactory as ShippingPointsCollectionFactory;
use Montonio\Shipping\Model\ShippingPointsFactory;
use Magento\Backend\App\Action;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\UrlInterface;
use Magento\Catalog\Model\Product\Attribute\Repository;

/**
 * @inheritDoc
 */
class Connection extends Action
{
    /**
     * @var MontonioSDK
     */
    protected $montonioSDK;

    /**
     * @var ShippingPointsCollectionFactory
     */
    protected $shippingPointsCollection;

    /**
     * @var ShippingPointsFactory
     */
    protected $shippingPointsFactory;

    /**
     * @var UrlInterface
     */
    protected $urlInterface;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var WriterInterface
     */
    protected $configWriter;

    /**
     * @var Context
     */
    protected $context;

    /**
     * @var Repository
     */
    protected $attributeRepository;

    /**
     * @var $messageManager
     */
    protected $messageManager;

    /**
     * @var $resultFactory
     */
    protected $resultFactory;

    /**
     * @var $_redirect
     */
    protected $_redirect;

    /**
     * Construct
     *
     * @param Context $context
     * @param MontonioSDK $montonioSDK
     * @param ShippingPointsCollectionFactory $shippingPointsCollection
     * @param UrlInterface $urlInterface
     * @param ShippingPointsFactory $shippingPointsFactory
     * @param Config $config
     * @param WriterInterface $configWriter
     * @param Repository $attributeRepository
     */
    public function __construct(
        Action\Context $context,
        MontonioSDK $montonioSDK,
        ShippingPointsCollectionFactory $shippingPointsCollection,
        UrlInterface $urlInterface,
        ShippingPointsFactory $shippingPointsFactory,
        Config $config,
        WriterInterface $configWriter,
        Repository $attributeRepository
    ) {
        parent::__construct($context);
        $this->montonioSDK = $montonioSDK;
        $this->shippingPointsCollection = $shippingPointsCollection;
        $this->shippingPointsFactory = $shippingPointsFactory;
        $this->urlInterface = $urlInterface;
        $this->config = $config;
        $this->configWriter = $configWriter;
        $this->attributeRepository = $attributeRepository;
        $this->context = $context;
    }

    /**
     * @inheritdoc
     */
    public function execute()
    {
        try {
            $methods = $this->retrieveApiData();

            $dataKey = sprintf(
                Config::DEFAULT_PATH_PATTERN,
                ShippingsConfigProvider::CODE,
                'montonio_methods/available_methods'
            );
            $this->setData($dataKey, json_encode($methods));
            $this->messageManager->addSuccessMessage(__('Update data succeeded.'));
            // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
        } catch (Exception $e) {
            /*
            * silence , we don't care if this fails
            */
            $message = 'Montonio Shipping was unable to sync pickup points in the background.
                        Please make sure your Montonio API credentials are correct.';
            $this->messageManager->addErrorMessage(__($message));

        }

        /** @var Redirect $result */
        $result = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        return $result->setUrl($this->_redirect->getRefererUrl());
    }

    /**
     * Retrieve payment methods API data.
     *
     * @return array
     */
    private function retrieveApiData(): array
    {
        $output = [];
        try {
            $methods = $this->montonioSDK->getPickupPoints();

            $oldPoints = $this->config->getValue('montonio_methods/available_methods') ?? '{}';
            $oldPoints = json_decode($oldPoints, true);

            foreach ($methods as $country => $methodsData) {

                $sort = ShippingsConfigProvider::DEFAULT_SORT_ORDER;
                foreach ($methodsData->providers as $providerName => $provider) {
                    $courier = false;

                    foreach ($provider as $type => $points) {
                        if (strpos($type, 'courier') !== false) {
                            $courier = true;
                        }
                        $valueKey = $country . '_' . $providerName . '_' . $type;
                        $items = [];
                        foreach ($points as $method) {
                            $items[] = [
                                'point_uuid' => $method->uuid,
                                'point_shipping_code' => $valueKey,
                                'point_name' => $method->name,
                                'point_region' => $method->region,
                                'point_locality' => $method->locality,
                            ];
                        }

                        try {
                            $this->updateShippingPoints($valueKey, $items);

                            $output[$valueKey] = [
                                'method_country' => $country,
                                'method_enabled' => $oldPoints[$valueKey]->method_enabled ?? 0,
                                'method_title' => $oldPoints[$valueKey]->method_title ?? $providerName,
                                'method_name' => $oldPoints[$valueKey]->method_name ?? $providerName . ' ' . $type,
                                'method_provider' => $providerName,
                                'method_type' => $type,
                                'method_price' => $oldPoints[$valueKey]->method_price ?? 0,
                                'method_free_shipping_enabled' =>
                                    $oldPoints[$valueKey]->method_free_shipping_enabled ?? 0,
                                'method_free_shipping_price' => $oldPoints[$valueKey]->method_free_shipping_price ?? 0,
                                'method_package_measurements_check_enabled' =>
                                    $oldPoints[$valueKey]->method_package_measurements_check_enabled ?? 0,
                                'method_package_maximum_weight' =>
                                    $oldPoints[$valueKey]->method_package_maximum_weight ?? 0,
                                'method_hide_no_measurements_enabled' =>
                                    $oldPoints[$valueKey]->method_hide_no_measurements_enabled ?? 0,
                                'sort_order' => $oldPoints[$valueKey]->sort_order ?? $sort
                            ];
                            // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
                        } catch (Exception $e) {
                            /*
                            * silence , we don't care if this fails
                            */
                        }
                        $sort = $sort + 10;
                    }
                    if (!$courier) {
                        $type = 'courier';
                        $valueKey = $country . '_' . $providerName . '_' . $type;
                        $output[$valueKey] = [
                            'method_country' => $country,
                            'method_enabled' => $oldPoints[$valueKey]->method_enabled ?? 0,
                            'method_title' => $oldPoints[$valueKey]->method_title ?? $providerName,
                            'method_name' => $oldPoints[$valueKey]->method_name ?? $providerName . ' ' . $type,
                            'method_provider' => $providerName,
                            'method_type' => $type,
                            'method_price' => $oldPoints[$valueKey]->method_price ?? 0,
                            'method_free_shipping_enabled' => $oldPoints[$valueKey]->method_free_shipping_enabled ?? 0,
                            'method_free_shipping_price' => $oldPoints[$valueKey]->method_free_shipping_price ?? 0,
                            'method_package_measurements_check_enabled' =>
                                $oldPoints[$valueKey]->method_package_measurements_check_enabled ?? 0,
                            'method_package_maximum_weight' =>
                                $oldPoints[$valueKey]->method_package_maximum_weight ?? 0,
                            'method_hide_no_measurements_enabled' =>
                                $oldPoints[$valueKey]->method_hide_no_measurements_enabled ?? 0,
                            'sort_order' => $oldPoints[$valueKey]->sort_order ?? $sort
                        ];
                    }
                }
            }
            // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
        } catch (Exception $e) {
            /*
            * silence , we don't care if this fails
            */
        }
        return $output;
    }

    /**
     * Set Config Data
     *
     * @param string $path
     * @param string $value
     * @return void
     */
    public function setData(string $path, string $value)
    {
        $this->configWriter->save($path, $value, $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT, $scopeId = 0);
    }

    /**
     * Update ShippingPoints
     *
     * @param string $shipping_code
     * @param array $points
     * @return void
     */
    public function updateShippingPoints(string $shipping_code, array $points)
    {
        $oldPoints = [];
        $parcelPoints = $this->shippingPointsCollection->create();
        $parcelPoints->addFieldToFilter('shipping_code', $shipping_code)->addFieldToSelect('*')->load();

        $collection = $parcelPoints->getItems();
        foreach ($collection as $item) {
            $oldPoints[$item->getUuid()] = $item->getUuid();
        }

        foreach ($points as $place) {
            $parcelPoint = $this->shippingPointsFactory->create();
            try {
                $parcelPoint->load($place['point_uuid']);
                if (isset($oldPoints[$parcelPoint->getUuid()])) {
                    unset($oldPoints[$parcelPoint->getUuid()]);
                }
                // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
            } catch (Exception $e) {
                /*
                * silence , we don't care if this fails
                */
            }

            if (!$parcelPoint || !$parcelPoint->getUuid()) {
                $parcelPoint->setUuid($place['point_uuid']);
            }

            $parcelPoint->setShippingCode($place['point_shipping_code']);
            $parcelPoint->setName($place['point_name']);
            $parcelPoint->setRegion($place['point_region'] ?? '');
            $parcelPoint->setLocality($place['point_locality']);
            try {
                $parcelPoint->save();
                // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
            } catch (Exception $e) {
                /*
                * silence , we don't care if this fails
                */
            }
        }

        foreach ($oldPoints as $point) {
            $parcelPoint = $this->shippingPointsFactory->create();
            $parcelPoint->load($point);
            $parcelPoint->delete();
        }
    }
}
