<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Controller\Shipping;

use Exception;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Request\Http;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Escaper;
use Magento\Sales\Api\Data\ShipmentTrackInterfaceFactory;
use Magento\Sales\Model\Convert\Order;
use Magento\Sales\Model\OrderFactory;
use Magento\Shipping\Model\ShipmentNotifier;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use Montonio\Shipping\Model\Carrier\MontonioShipping;
use Psr\Log\LoggerInterface;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;
use Magento\Sales\Api\ShipmentRepositoryInterface;

/**
 * @inheritDoc
 */
class Webhook extends Action implements CsrfAwareActionInterface
{
    /**
     * @const CARRIER_CODE
     */
    public const CARRIER_CODE = 'montonio_shipping';

    /**
     * @const CARRIER_TITLE
     */
    public const CARRIER_TITLE = 'Montonio Shipping';

    /**
     * @var Http
     */
    protected $request;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var OrderFactory
     */
    protected $orderFactory;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var Order
     */
    protected $orderConverter;

    /**
     * @var ShipmentNotifier
     */
    protected $shipmentNotifier;

    /**
     * @var ShipmentTrackInterfaceFactory
     */
    protected $trackFactory;

    /**
     * @var ShipmentRepositoryInterface
     */
    protected $shipmentRepository;

    /**
     * @var Escaper
     */
    protected $escaper;

    /**
     * @var JsonFactory
     */
    protected $jsonResultFactory;

    /**
     * Construct
     *
     * @param Context $context
     * @param Http $request
     * @param OrderFactory $orderFactory
     * @param ShipmentNotifier $shipmentNotifier
     * @param Order $orderConverter
     * @param ShipmentTrackInterfaceFactory $trackFactory
     * @param Config $config
     * @param LoggerInterface $logger
     * @param ShipmentRepositoryInterface $shipmentRepositoryInterface
     * @param Escaper $escaper
     * @param JsonFactory $jsonResultFactory
     */
    public function __construct(
        Context $context,
        Http $request,
        OrderFactory $orderFactory,
        ShipmentNotifier $shipmentNotifier,
        Order $orderConverter,
        ShipmentTrackInterfaceFactory $trackFactory,
        Config $config,
        LoggerInterface $logger,
        ShipmentRepositoryInterface $shipmentRepositoryInterface,
        Escaper $escaper,
        JsonFactory $jsonResultFactory
    ) {

        $this->request = $request;
        $this->config = $config;
        $this->orderFactory = $orderFactory;
        $this->logger = $logger;
        $this->orderConverter = $orderConverter;
        $this->shipmentNotifier = $shipmentNotifier;
        $this->trackFactory = $trackFactory;
        $this->shipmentRepository = $shipmentRepositoryInterface;
        $this->escaper = $escaper;
        $this->jsonResultFactory = $jsonResultFactory;
        parent::__construct($context);
    }

    /**
     * @inheritDoc
     */
    public function createCsrfValidationException(
        RequestInterface $request
    ): ?InvalidRequestException {
        return null;
    }

    /**
     * @inheritDoc
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }

    /**
     * @inheritdoc
     */
    public function execute()
    {
        $token = $this->request->getParam('webhook_token') ?? false;

        $result = $this->jsonResultFactory->create();

        if (!$token) {
            $this->logger->info('no token');
            $HTTP_BAD_REQUEST = \Magento\Framework\Webapi\Exception::HTTP_BAD_REQUEST;
            $result->setHttpResponseCode($HTTP_BAD_REQUEST);
            $result->setData(['response_message' => __($HTTP_BAD_REQUEST)]);
            return $result;
        }

        $environment = $this->config->getValue('environment');

        $HTTP_UNAUTHORIZED = \Magento\Framework\Webapi\Exception::HTTP_UNAUTHORIZED;
        try {
            $response = $this->config->decodeSignature(
                $token,
                $this->config->getValue($environment.'_private_key')
            );
        } catch (Exception $exception) {
            $this->logger->info('no signature: '. $exception->getMessage());
            $this->logger->info($token);
            $result->setHttpResponseCode($HTTP_UNAUTHORIZED);
            $result->setData(['response_message' => __($HTTP_UNAUTHORIZED)]);
            return $result;
        }

        try {
            $this->handleShipmentCreatedWebhook($response);
        } catch (Exception $exception) {
            $this->logger->info($exception->getMessage());
            $this->logger->info($token);
            $result->setHttpResponseCode((string)$HTTP_UNAUTHORIZED);
            $result->setData(['response_message' => __((string)$HTTP_UNAUTHORIZED)]);
            return $result;
        }

        $result->setData(['response_message' => __('ok')]);
        return $result;
    }

    /**
     * Handle Shipment Webhook
     *
     * @param  $response
     * @return void
     */
    protected function handleShipmentCreatedWebhook($response): void
    {

        $shipmentId = $response->merchant_reference ?? false;

        if ($shipmentId) {
            //hooks can fire too fast when order is not processed properly from merchant side yet
            sleep(10);

            $shipmentId = explode('_', $shipmentId)['1'];

            $trackData = $response->tracking_numbers ?? [];

            try {

                $shipment = $this->shipmentRepository->get((int)$shipmentId);
                $newTracks = [];

                //detect if shipment exists
                if ($shipment->getIncrementId() == $shipmentId && !empty($trackData)) {

                    //load tracks
                    $tracks = $shipment->getAllTracks() ?? [];

                    //if tracks
                    if (!empty($tracks)) {
                        //clap along
                        $i = 0;
                        //go over tracks
                        foreach ($trackData as $key => $requestedTrack) {
                            $existingTracks = $this->isExistingTrack($tracks, $requestedTrack);
                            //if we have existing we try to update
                            if (!empty($existingTracks)) {
                                $addComment = false;
                                foreach ($existingTracks as $existingTrack) {
                                    $trackLink = $this->escaper->escapeUrl($requestedTrack->link);
                                    //update only if something changes
                                    if ($existingTrack['track']->getDescription() !== $trackLink) {
                                        $existingTrack['track']->setDescription($trackLink);
                                        //set it back to track array
                                        $tracks[$existingTrack['track_key']] = $existingTrack['track'];
                                        $addComment = true;
                                    }
                                }

                                if ($addComment) {
                                    //set changed tracks back to shipping object
                                    $shipment->addComment(__('Updated track information from webhook response.'));
                                }

                            } else {
                                //if we have tracks that do not exist yet store for later
                                $newTracks[] = $requestedTrack;
                            }
                            $i = $i++;
                        }

                        $shipment->setTracs($tracks);

                    //if no tracks yet
                    } else {
                        //set a comment to order
                        $shipment->addComment(__('Added tracking information from webhook response.'));
                        foreach ($trackData as $key => $requestedTrack) {
                            $shipment = $this->addTrack($shipment, $requestedTrack);
                        }
                    }

                    //if we have new tracks
                    if (!empty($newTracks)) {
                        $shipment->addComment(__('Added tracking information from webhook response.'));
                        foreach ($newTracks as $key => $requestedTrack) {
                            $shipment = $this->addTrack($shipment, $requestedTrack);
                        }
                    }

                    $this->shipmentRepository->save($shipment);

                }

            } catch (Exception $e) {
                $this->logger->info($e->getMessage());
            }
        }
    }

    /**
     * Add new tracking objects to shipment object
     *
     * @param  $shipment
     * @param  $requestedTrack
     * @return mixed
     */
    public function addTrack($shipment, $requestedTrack)
    {
        return $shipment->addTrack(
            $this->trackFactory->create()
                ->setNumber($requestedTrack->code)
                ->setDescription($requestedTrack->link)
                ->setCarrierCode(MontonioShipping::CARRIER_CODE)
                ->setTitle(MontonioShipping::CARRIER_TITLE)
        );
    }

    /**
     * See if track is already existing in tracks array
     *
     * @param array $tracks
     * @param $requestedTrack
     * @return array
     */
    private function isExistingTrack(array $tracks, $requestedTrack): array
    {
        $result = [];

        foreach ($tracks as $k => $track) {

            if ($requestedTrack->code === $track->getTrackNumber()) {
                $result[$k] = ['track_key' => $k, 'track' => $track ];
            }
        }

        return  $result;
    }
}
