<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Gateway\Export;

use Magento\Sales\Model\Order\Shipment;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use Montonio\Shipping\Gateway\Shippings\MontonioSDK;
use Psr\Log\LoggerInterface;
use Magento\Framework\UrlInterface;

class ShipmentExport
{
    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var MontonioSDK
     */
    protected $montonioSDK;
    /**
     * @var UrlInterface
     */
    protected $urlInterface;

    /**
     * Construct
     *
     * @param LoggerInterface $logger
     * @param Config $config
     * @param MontonioSDK $montonioSDK
     * @param UrlInterface $urlInterface
     */
    public function __construct(
        LoggerInterface $logger,
        Config $config,
        MontonioSDK $montonioSDK,
        UrlInterface $urlInterface
    ) {
        $this->logger = $logger;
        $this->config = $config;
        $this->montonioSDK = $montonioSDK;
        $this->urlInterface = $urlInterface;
    }

    /**
     * Export
     *
     * @param Shipment $shipment
     * @param bool $async
     * @return bool $result
     */
    public function export(Shipment $shipment, bool $async = false) : bool
    {
        $data = $this->getCreateShipmentData($shipment);
        $result = false;

        try {
            //this will hit false here because we will need the response data until hook data is not in sync
            $result = $this->montonioSDK->postShipment($data, $async);
            if ($result !== false) {
                $merchantRef = $shipment->getOrder()->getIncrementId() . '_' . $shipment->getIncrementId();
                $comment = __(
                    'Shipment %1 with merchant reference %2 was successfully exported to Montonio',
                    $shipment->getIncrementId(),
                    $merchantRef
                );
            } else {
                $comment = __('Error occurred while exporting shipment %1 to Montonio', $shipment->getIncrementId());
            }
        } catch (Exception $e) {
            $comment = __(
                'Error occurred while exporting shipment %1 to Montonio: %2',
                $shipment->getIncrementId(),
                $e->getMessage()
            );
        }

        //we store the parts of response in order for hook to add full data needed for tracking
        $shipment->getOrder()->addStatusHistoryComment($comment)->setIsCustomerNotified(null);

        return (bool)$result;
    }

    /**
     * Get Shipment Data as array
     *
     * @param Shipment $shipment
     * @return array
     */
    protected function getCreateShipmentData(Shipment $shipment) : array
    {
        $order = $shipment->getOrder();

        $billingAddress = $order->getBillingAddress();
        $shippingAddress = $order->getShippingAddress();

        //get the WHATEVER_AFTER_THAT as string after 3rd _ as in montonio_shipping_CONTRYCODE_WHATEVER_AFTER_THAT
        $shippingMethodIdentifier = explode('_', $order->getShippingMethod(), '4');
        $shippingMethodIdentifier = end($shippingMethodIdentifier);

        //street is a string or array
        $bStreet = is_array($bStreet = $billingAddress->getStreet()) ? implode(PHP_EOL, $bStreet) : $bStreet;
        $sStreet = is_array($sStreet = $shippingAddress->getStreet()) ? implode(PHP_EOL, $sStreet) : $sStreet;

        $data = [
            'sender_name'               => (string) $this->config->getValue('sender_info/sender_name'),
            'sender_phone_number'       => (string) $this->config->getValue('sender_info/sender_phone'),
            'sender_street_address_1'   => (string) $this->config->getValue('sender_info/sender_street_address_1'),
            'sender_locality'           => (string) $this->config->getValue('sender_info/sender_locality'),
            'sender_postal_code'        => (string) $this->config->getValue('sender_info/sender_postal_code'),
            'sender_region'             => (string) $this->config->getValue('sender_info/sender_region'),
            'sender_country'            => (string) $this->config->getValue('sender_info/sender_country'),
            'merchant_reference'        => (string) $order->getIncrementId(). '_' . $shipment->getIncrementId(),
            'webhook_url'               => $this->getWebhookUrl(),
            'shipping_method'           => (string) $shippingMethodIdentifier,
            'billing_first_name'        => (string) $billingAddress->getFirstname(),
            'billing_last_name'         => (string) $billingAddress->getLastname(),
            'billing_company'           => (string) $billingAddress->getCompany(),
            'billing_street_address_1'  => (string) $bStreet,
            'billing_locality'          => (string) $billingAddress->getCity(),
            'billing_postal_code'       => (string) $billingAddress->getPostcode(),
            'billing_email'             => (string) $billingAddress->getEmail(),
            'billing_phone_number'      => (string) $shippingAddress->getTelephone(),
            'shipping_first_name'       => (string) $shippingAddress->getFirstname(),
            'shipping_last_name'        => (string) $shippingAddress->getLastname(),
            'shipping_company'          => (string) $shippingAddress->getCompany(),
            'shipping_street_address_1' => (string) $sStreet,
            'shipping_locality'         => (string) $shippingAddress->getCity(),
            'shipping_postal_code'      => (string) $shippingAddress->getPostcode(),
            'shipping_email'            => (string) $shippingAddress->getEmail(),
            'shipping_phone_number'     => (string) $shippingAddress->getTelephone()
        ];

        if (strpos($shippingMethodIdentifier, 'courier') === false) {
            $data['pickup_point_uuid'] = $order->getMontonioShippingPoint();
        } else {
            $data['shipping_country'] = (string) $shippingAddress->getCountryId();
            $data['billing_country'] = (string) $billingAddress->getCountryId();
        }

        $parcelWeight = 0;

        foreach ($shipment->getItems() as $item) {
            //we need only single simple, configurable and bundle parent items
            if (isset($item->getData('product_options')['info_buyRequest']['product']) && $item->getProductId() !==
                $item->getData('product_options')['info_buyRequest']['product']
            ) {
                continue;
            }

            $parcelWeight += (float) $item->getWeight() * (int) $item->getQty() * $this->config->getWeightMultiplier();
        }

        $data['parcels'] = [
            [
                'weight' => $parcelWeight
            ]
        ];

        return $data;
    }

    /**
     * Get webhook url
     *
     * @return string
     */
    public function getWebhookUrl(): string
    {
        return $this->urlInterface->getBaseUrl() . 'montonio/shipping/webhook';
    }
}
