<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

declare(strict_types=1);
namespace Montonio\Shipping\Gateway\Shippings\Config;

use Exception;
use Firebase\JWT\JWT;
use Firebase\JWT\Key;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Escaper;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Store\Model\ScopeInterface;
use Montonio\Shipping\Model\Carrier\MontonioShipping;
use Montonio\Shipping\Model\ResourceModel\ShippingPoints\CollectionFactory as ShippingPointsCollectionFactory;
use Montonio\Shipping\Model\Shipping\ShippingsConfigProvider;
use Psr\Log\LoggerInterface;
use Zend_Db_Expr;

/**
 * @inheritdoc
 */
class Config
{

    /**
     * @var string
     */
    public const DEFAULT_PATH_PATTERN = 'carriers/%s/%s';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var Json
     */
    protected $json;

    /**
     * @var Escaper
     */
    protected $escaper;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var mixed|string|null
     */
    protected $methodCode;

    /**
     * @var mixed|string
     */
    protected $pathPattern;

    /**
     * @var ShippingPointsCollectionFactory
     */
    protected $shippingPointsCollection;

    /**
     * Construct
     *
     * @param ScopeConfigInterface $scopeConfig
     * @param Escaper $escaper
     * @param Json $json
     * @param LoggerInterface $logger
     * @param ShippingPointsCollectionFactory $shippingPointsCollection
     * @param mixed $methodCode
     * @param string $pathPattern
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Escaper $escaper,
        Json $json,
        LoggerInterface $logger,
        ShippingPointsCollectionFactory $shippingPointsCollection,
        $methodCode = null,
        $pathPattern = self::DEFAULT_PATH_PATTERN
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->escaper = $escaper;
        $this->json = $json;
        $this->logger = $logger;

        //if we DI this class directly to our other components we need this to be initiated
        //in all other case it is initiated trough di.xml
        if (!$methodCode) {
            $methodCode = ShippingsConfigProvider::CODE;
        }

        $this->methodCode = $methodCode;
        $this->pathPattern = $pathPattern;
        $this->shippingPointsCollection = $shippingPointsCollection;
    }

    /**
     * Retrieve information from shipping configuration
     *
     * @param string $field
     * @param int|null $storeId
     *
     * @return mixed
     */
    public function getValue(string $field, $storeId = null)
    {
        if ($this->methodCode === null || $this->pathPattern === null) {
            return null;
        }

        return $this->scopeConfig->getValue(
            sprintf($this->pathPattern, $this->methodCode, $field),
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * Get config isActive
     *
     * @param mixed $storeId
     * @return bool
     */
    public function isActive($storeId = null)
    {
        return $this->getValue('active', $storeId);
    }

    /**
     * Get methodCode
     *
     * @return string
     */
    public function getCode()
    {
        return $this->methodCode;
    }

    /**
     * Retrieve title display option.
     *
     * @return string
     */
    public function getTitleDisplayOption(): string
    {
        return $this->escaper->escapeHtml((string)$this->getValue('title_display_option'));
    }

    /**
     * Retrieve assoc array of checkout configuration
     *
     * @return array
     */
    public function getConfig()
    {
        //we do not use this and return empty array
        return [];
    }

    /**
     * Generate HS256 signature
     *
     * @param array $data
     * @param string $key
     * @param boolean $jsonEncode
     * @return mixed
     */
    public function generateSignature(array $data, string $key, bool $jsonEncode = false)
    {
        if ($jsonEncode) {
            $data = json_encode($data);
        }
        return JWT::encode($data, $key, 'HS256');
    }

    /**
     * Decode HS256 signature
     *
     * @param string $data
     * @param string $key
     * @return array|string
     */
    public function decodeSignature(string $data, string $key)
    {
        return JWT::decode($data, new Key($key, 'HS256'));
    }

    /**
     * Get Available Methods
     *
     * @param string $country
     * @param mixed $storeId
     * @return array
     */
    public function getAvailableMethods(string $country = '', $storeId = null): array
    {

        $methods = [];
        try {
            $values = $this->json->unserialize($this->getValue('montonio_methods/available_methods', $storeId));
            $methods = array_flip(array_keys($values));

            foreach ($methods as $key => $value) {
                if (is_array($values[$key])) {
                    $methods[$key] = $values[$key];
                }

                if (is_array($values[$key]) && !$values[$key]['method_enabled']) {
                    unset($methods[$key]);
                }
            }
        } catch (Exception $e) {
            $this->logger->critical($e);
        }

        return $methods;
    }

    /**
     * Get All Methods
     *
     * @return array
     */
    public function getAllMethods(): array
    {

        $methods = [];
        try {
            $values = $this->json->unserialize($this->getValue('montonio_methods/available_methods'));
            $methods = array_flip(array_keys($values));

            foreach ($methods as $key => $value) {
                if (is_array($values[$key])) {
                    $methods[$key] = $values[$key];
                }

            }
        } catch (Exception $e) {
            $this->logger->critical($e);
        }

        return $methods;
    }

    /**
     * Return appropriate multiplier to convert to kg
     *
     * @return float|int
     */
    public function getWeightMultiplier()
    {
        $weightUnit = $this->scopeConfig->getValue(
            'general/locale/weight_unit',
            ScopeInterface::SCOPE_STORE
        );

        switch ($weightUnit) {
            case 'kgs':
                return 1;
            case 'g':
                return 0.001;
            case 'lbs':
                return 0.45;
            case 'oz':
                return 0.028;
            default:
                // Unknown unit, treat as missing measurements
                return 0;
        }
    }

    /**
     * Check if order has Montonio Shipping method
     *
     * @param string $shippingMethodCode
     * @return bool
     */
    public function matchShippingMethod(string $shippingMethodCode): bool
    {

        $found = strpos(
            $shippingMethodCode,
            MontonioShipping::CARRIER_CODE
        );

        return $found !== false && $found >= 0;
    }

    /**
     * Get Enabled shipping point data as Array
     *
     * @return array
     */
    public function getShippingPoints(): array
    {
        $parcelPoints = $this->shippingPointsCollection->create();
        $parcelPoints
            ->addFieldToSelect('*')
            ->addFieldToFilter('shipping_code', ['in' => array_keys($this->getAvailableMethods())]);

        $zendDbExpr = new Zend_Db_Expr(
            'shipping_code ASC, FIELD(locality, 
            "Pärnu vald", "Pärnu","Pärnu linn", "Tartu vald", "Tartu", "Tartu linn", "Tallinn") DESC, 
            locality asc, region asc, name ASC'
        );
        $parcelPoints
            ->getSelect()
            ->order($zendDbExpr);
        $parcelPoints->load();
        $points = [];
        $collection = $parcelPoints->getItems();
        foreach ($collection as $item) {
            $var = !empty($item->getData('locality')) ? $item->getData('locality') : $item->getData('region');
            $points[$item->getData('shipping_code')][$var][$item->getData('uuid')] = $item->getData('name');
        }

        return [
            'points' => $points
        ];
    }
}
