<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Shipping\Gateway\Shippings;

use Exception;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use GuzzleHttp\Client as ClientFactory;
use Psr\Log\LoggerInterface;

/**
 * Class MontonioSDK for requesting available methods from gateway
 */
class MontonioSDK
{

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var TimezoneInterface
     */
    protected $timezoneInterface;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var ClientFactory
     */
    protected $httpClient;

    /**
     * Construct
     *
     * @param Config $config
     * @param TimezoneInterface $timezoneInterface
     * @param LoggerInterface $logger
     * @param ClientFactory $httpClient
     */
    public function __construct(
        Config            $config,
        TimezoneInterface $timezoneInterface,
        LoggerInterface   $logger,
        ClientFactory $httpClient
    ) {
        $this->config = $config;
        $this->timezoneInterface = $timezoneInterface;
        $this->logger = $logger;
        $this->httpClient = $httpClient;
    }

    /**
     * Get BearerToken
     *
     * @return string
     */
    protected function getBearerToken()
    {
        $time = $this->timezoneInterface->date()->format('U');
        $environment = $this->config->getValue('environment');
        $data = [
            'access_key' => $this->config->getValue($environment.'_public_key'),
            'iat'        => $time,
            'exp'        => $time + (60 * 10)
        ];

        return $this->config->generateSignature(
            $data,
            $this->config->getValue($environment.'_private_key')
        );
    }

    /**
     * Get PickupPoints as array
     *
     * @return mixed
     */
    public function getPickupPoints()
    {
        $route = '/pickup-points';
        $options = [
            'http' => [
                'header' => ['Content-Type' => 'application/json',
                    'Authorization' => 'Bearer ' . $this->getBearerToken()],
                'method' => 'GET'
            ]
        ];
        return $this->apiRequest($route, $options);
    }

    /**
     * Post shipment data
     *
     * @param array $data
     * @param bool $async
     * @return mixed
     */
    public function postShipment($data, $async = false)
    {
        $route = $async ? '/shipments/create-async' : '/shipments';
        $options = [
            'http' => [
                'header' => ['Content-Type' => 'application/json',
                    'Authorization' => 'Bearer ' . $this->getBearerToken()],
                'method' => 'POST',
                'content' => $data
            ]
        ];

        return $this->apiRequest($route, $options);
    }

    /**
     * Register Tracking Webhook
     *
     * @param string $url
     * @return mixed
     */
    public function registerTrackingWebhook($url)
    {
        //$route = '/stores/register-tracking-webhook';
        $route = '/stores/store-webhook';

        $options = [
            'http' => [
                'header' => ['Content-Type' => 'application/json',
                    'Authorization' => 'Bearer ' . $this->getBearerToken()],
                'method' => 'PATCH',
                'content' => [
                    'tracking_webhook_url' => $url
                ]
            ]
        ];

        return $this->apiRequest($route, $options);
    }

    /**
     * Create Labels
     *
     * @param array $data
     * @return mixed
     */
    public function createLabels($data)
    {
        $route = '/shipments/label-from-store';
        $options = [
            'http' => [
                'header' => ['Content-Type' => 'application/json',
                    'Authorization' => 'Bearer ' . $this->getBearerToken()],
                'method' => 'POST',
                'content' => $data
            ]
        ];

        return $this->ApiRequest($route, $options);
    }

    /**
     * Function for making API calls with file_get_contents
     *
     * @param string $apiEndpoint
     * @param array $options
     * @return mixed
     */
    protected function apiRequest($apiEndpoint, $options)
    {

        $environment = $this->config->getValue('environment');
        $apiUrl = $this->config->getValue($environment.'_api_url').$apiEndpoint;

        try {
            $client = $this->httpClient;

            $payload = [
                'allow_redirects' => false,
                'connect_timeout' => 30,
                'timeout' => 30,
                'headers' => $options['http']['header'] ?? []
            ];

            if (isset($options['http']['content'])) {
                $payload['json'] = $options['http']['content'];
            }

            $request = $client->request($options['http']['method'], $apiUrl, $payload);
            $response = json_decode($request->getBody()->getContents());

            if (strstr($apiEndpoint, 'create-async')) {
                //for async request we don't get response, so we assume success
                //truth arrives with webhook hit later
                $response = true;
            }

        } catch (Exception $e) {
            $this->logger->critical($e);
            throw new LocalizedException(__($e->getMessage()));
        }

        return $response;
    }
}
