<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Model\Carrier;

use Magento\Catalog\Api\ProductAttributeRepositoryInterface;
use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObject;
use Magento\Framework\DataObjectFactory;
use Magento\Framework\Escaper;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\View\Asset\Repository;
use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory;
use Magento\Quote\Model\Quote\Address\RateResult\MethodFactory;
use Magento\Sales\Api\Data\ShipmentTrackInterface;
use Magento\Sales\Api\ShipmentTrackRepositoryInterface;
use Magento\Shipping\Model\Carrier\AbstractCarrier;
use Magento\Shipping\Model\Carrier\CarrierInterface;
use Magento\Shipping\Model\Rate\Result;
use Magento\Shipping\Model\Rate\ResultFactory;
use Montonio\Shipping\Gateway\Shippings\Config\Config;

use Psr\Log\LoggerInterface;

class MontonioShipping extends AbstractCarrier implements CarrierInterface
{
    /**
     * @var string
     */
    public const NO_MONTONIO_SHIPPING_PRODUCT_ATTRIBUTE = 'nomontonioshipping';

    /**
     * @var string
     */
    public const CARRIER_CODE = 'montonio_shipping';

    /**
     * @var string
     */
    public const CARRIER_TITLE = 'Montonio tracking info';

    /**
     * @var string
     */
    public const COURIER_SUFFIX = 'courier';

    /**
     * @var string
     */
    protected $_code = self::CARRIER_CODE;

    /**
     * @var ResultFactory
     */
    protected $rateResultFactory;

    /**
     * @var ErrorFactory
     */
    protected $rateErrorFactory;

    /**
     * @var MethodFactory
     */
    protected $rateMethodFactory;

    /**
     * @var Repository
     */
    protected $assetRepo;

    /**
     * @var Escaper
     */
    protected $escaper;

    /**
     * @var Result
     */
    protected $rateResult;

    /**
     * @var ProductRepository
     */
    protected $productRepository;

    /**
     * @var ProductAttributeRepositoryInterface
     */
    protected $attributeRepository;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var ShipmentTrackRepositoryInterface
     */
    protected $shipmentTrackRepositoryInterface;

    /**
     * @var array
     */
    protected $data;

    /**
     * @var FilterBuilder
     */
    protected $filterBuilder;

    /**
     * @var SearchCriteriaBuilder
     */
    protected $criteriaBuilder;

    /**
     * @var DataObjectFactory
     */
    protected $dataObjectFactory;

    /**
     * Construct
     *
     * @param ScopeConfigInterface $scopeConfig
     * @param ErrorFactory $rateErrorFactory
     * @param LoggerInterface $logger
     * @param Config $config
     * @param ResultFactory $rateResultFactory
     * @param MethodFactory $rateMethodFactory
     * @param Result $rateResult
     * @param Repository $assetRepo
     * @param ProductRepository $productRepository
     * @param ProductAttributeRepositoryInterface $attributeRepository
     * @param Escaper $escaper
     * @param ShipmentTrackRepositoryInterface $shipmentTrackRepositoryInterface
     * @param FilterBuilder $filterBuilder
     * @param SearchCriteriaBuilder $searchCriteria
     * @param DataObjectFactory $dataObjectFactory
     * @param array $data
     */
    public function __construct(
        ScopeConfigInterface                $scopeConfig,
        ErrorFactory                        $rateErrorFactory,
        LoggerInterface                     $logger,
        Config                              $config,
        ResultFactory                       $rateResultFactory,
        MethodFactory                       $rateMethodFactory,
        Result                              $rateResult,
        Repository                          $assetRepo,
        ProductRepository                   $productRepository,
        ProductAttributeRepositoryInterface $attributeRepository,
        Escaper                             $escaper,
        ShipmentTrackRepositoryInterface                   $shipmentTrackRepositoryInterface,
        FilterBuilder                                              $filterBuilder,
        SearchCriteriaBuilder                                      $searchCriteria,
        DataObjectFactory                                          $dataObjectFactory,
        array                                                      $data = []
    ) {
        $this->rateResultFactory = $rateResultFactory;
        $this->rateMethodFactory = $rateMethodFactory;
        $this->rateErrorFactory = $rateErrorFactory;
        $this->assetRepo = $assetRepo;
        $this->escaper = $escaper;
        $this->rateResult = $rateResult;
        $this->productRepository = $productRepository;
        $this->attributeRepository = $attributeRepository;

        $this->config = $config;
        $this->scopeConfig = $scopeConfig;
        $this->logger = $logger;
        $this->shipmentTrackRepositoryInterface = $shipmentTrackRepositoryInterface;
        $this->filterBuilder = $filterBuilder;
        $this->criteriaBuilder = $searchCriteria;
        $this->dataObjectFactory = $dataObjectFactory;
        $this->data = $data;

        $this->_initAttributeOptions(self::NO_MONTONIO_SHIPPING_PRODUCT_ATTRIBUTE);

        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
    }

    /**
     * Get allowed methods
     *
     * @return array
     */
    public function getAllowedMethods()
    {
        $methods = [];
        $montonioMethods = $this->config->getAvailableMethods();
        foreach ($montonioMethods as $key => $method) {
            $methods[$key] = $method['method_title'];
        }
        return $methods;
    }

    /**
     * @inheritDoc
     */
    public function collectRates(RateRequest $request)
    {
        $hideMethod = false;
        $noMontonioShipping = [];

        if (!$this->config->isActive()) {
            return false;
        }

        $countryId = $request->getDestCountryId();
        $storeId = $request->getStoreId();

        $montonioMethods = $this->config->getAvailableMethods($countryId, $storeId);
        $result = $this->rateResultFactory->create();

        $items = $request->getAllItems();
        $weight = 0;

        // check for excluded products
        $isDenyCheckEnabled = $this->config->getValue('deny_check_enable');

        foreach ($items as $item) {
            $weight += $item->getWeight();
            if ($item->getWeight() <= 0) {
                $hideMethod = true;
            }

            $product = $this->productRepository->getById($item->getProduct()->getId());

            $data = $product->getData(self::NO_MONTONIO_SHIPPING_PRODUCT_ATTRIBUTE);
            if ($data) {
                // phpcs:ignore Magento2.Performance.ForeachArrayMerge
                $noMontonioShipping = array_merge($noMontonioShipping, array_values(explode(',', $data)));
            }
        }

        foreach ($montonioMethods as $key => $method) {
            $shippingPrice = $method['method_price'];
            $isMeasurementsCheckEnabled = $method['method_package_measurements_check_enabled'];

            if ($isMeasurementsCheckEnabled) {
                $maximumWeight = $method['method_package_maximum_weight'];
                if ($weight > $maximumWeight) {
                    continue;
                }
            }

            $isHideNoMeasurementsCheckEnabled = $method['method_hide_no_measurements_enabled'];

            if ($isHideNoMeasurementsCheckEnabled) {
                if ($hideMethod) {
                    continue;
                }
            }

            if ($isDenyCheckEnabled) {
                if (isset($this->denyOptions[$key]) && in_array($this->denyOptions[$key], $noMontonioShipping)) {
                    continue;
                }
            }

            $isFreeShippingEnabled = $method['method_free_shipping_enabled'];
            $packageValue = $request->getPackageValueWithDiscount();
            if (empty($shippingPrice) || !is_numeric($shippingPrice) || $shippingPrice < 0) {
                $shippingPrice = '0.00';
            }

            if ($isFreeShippingEnabled && $packageValue >= $method['method_free_shipping_price']) {
                $shippingPrice = '0.00';
            }
            $shippingMethod = $this->rateMethodFactory->create();

            $shippingMethod->setCarrier($this->_code);
            $shippingMethod->setCarrierTitle($method['method_title']);
            $shippingMethod->setMethod($key);
            $shippingMethod->setMethodTitle($method['method_name']);
            $shippingMethod->setPrice($shippingPrice);
            $shippingMethod->setCost($shippingPrice);

            $result->append($shippingMethod);
        }

        return $result;
    }

    /**
     * Get Image full path
     *
     * @param array $method
     * @return string
     */
    public function getImageFullPath(array $method)
    {
        $fileId = 'Montonio_Shipping::images/'.$method['method_provider'].'.png';

        $params = [
            'area' => 'frontend' //for admin area its backend
        ];

        $asset = $this->assetRepo->createAsset($fileId, $params);

        $imageFullPath = null;
        try {
            $imageFullPath = $asset->getSourceUrl();
            // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
        } catch (Exception $exception) {
            /*
            * silence , we don't care if this fails
            */
        }

        return $imageFullPath;
    }

    /**
     * Get Tracking info from tracking record
     *
     * @param string $number
     * @return DataObject|null
     */
    public function getTrackingInfo(string $number)
    {

        //ShipmentTrackInterface::TRACK_NUMBER
        $this->criteriaBuilder
            ->addFilters([
                    $this->filterBuilder
                        ->setField(ShipmentTrackInterface::TRACK_NUMBER)
                        ->setValue($number)
                        ->setConditionType('eq')
                        ->create()
                ])
            ->setPageSize(20);

        $searchCriteria = $this->criteriaBuilder->create();
        $track = $this->shipmentTrackRepositoryInterface->getList($searchCriteria)->getFirstItem();

        $trackInformationObject = null;

        if (!empty($track)) {
            $trackInformationObject = $this->dataObjectFactory->create();
            $trackInformationObject->setTracking($track->getTrackNumber())
                ->setUrl($track->getDescription())
                ->setCarrierTitle($track->getTitle());
        }

        return  $trackInformationObject;
    }

    /**
     * Init attribute options
     *
     * @param string $attrCode
     * @return void
     */
    private function _initAttributeOptions(string $attrCode = '')
    {
        if (empty($this->denyOptions)) {
            $this->denyOptions = [];
            if ($attr = $this->_getAttribute($attrCode)) {
                $options = $attr->setStoreId(0)->getOptions();
                foreach ($options as $option) {
                    $this->denyOptions[$option['label']] = $option['value'];
                }
            }
        }
    }

    /**
     * Get Magento Attribute Object
     *
     * @param string $attrCode
     * @return mixed
     */
    private function _getAttribute(string $attrCode)
    {
        $result = false;
        try {
            $result = $this->attributeRepository->get($attrCode);
            // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
        } catch (NoSuchEntityException $e) {
            /*
            * silence , we don't care if this fails
            */
        }

        return $result;
    }
}
