<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Model;

use Exception;
use Montonio\Shipping\Gateway\Shippings\MontonioSDK;
use Montonio\Shipping\Model\ResourceModel\ShippingPoints\CollectionFactory as ShippingPointsCollectionFactory;

class Cron
{
    /**
     * @var MontonioSDK
     */
    protected $montonioSDK;

    /**
     * @var ShippingPoinsCollectionFactory
     */
    protected $shippingPointsCollection;

    /**
     * @var null|ShippingPointsFactory
     */
    protected $shippingPointsFactory = null;

    /**
     * Construct function.
     *
     * @param MontonioSDK $montonioSDK
     * @param ShippingPointsCollectionFactory $shippingPointsCollection
     * @param ShippingPointsFactory $shippingPointsFactory
     */
    public function __construct(
        MontonioSDK $montonioSDK,
        ShippingPointsCollectionFactory $shippingPointsCollection,
        ShippingPointsFactory $shippingPointsFactory
    ) {
        $this->montonioSDK = $montonioSDK;
        $this->shippingPointsCollection = $shippingPointsCollection;
        $this->shippingPointsFactory = $shippingPointsFactory;
    }

    /**
     * @inheritDoc
     */
    public function execute()
    {
        try {
            $methods = $this->montonioSDK->getPickupPoints();

            foreach ($methods as $country => $methodsData) {
                foreach ($methodsData->providers as $providerName => $provider) {
                    foreach ($provider as $type => $methods) {

                        $points = [];
                        foreach ($methods as $method) {
                            $points[] = [
                                'point_uuid' => $method->uuid,
                                'point_shipping_code' => $country . '_' . $providerName . '_' . $type,
                                'point_name' => $method->name,
                                'point_region' => $method->region ?? '',
                                'point_locality' => $method->locality,
                            ];
                        }

                        try {
                            $this->updateShippingPoints($country . '_' . $providerName . '_' . $type, $points);

                            // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
                        } catch (Exception $e) {
                            /*
                            * silence , we don't care if this fails
                            */
                        }
                    }
                }
            }

            // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
        } catch (Exception $e) {
            /*
            * silence , we don't care if this fails
            */
        }
    }

    /**
     * Update shipping points
     *
     * @param string $shipping_code
     * @param array $points
     * @return void
     */
    public function updateShippingPoints(string $shipping_code, array $points)
    {
        $oldPoints = [];
        $parcelPoints = $this->shippingPointsCollection->create();
        $parcelPoints->addFieldToFilter('shipping_code', $shipping_code)->addFieldToSelect('*')->load();

        $collection = $parcelPoints->getItems();
        foreach ($collection as $item) {
            $oldPoints[$item->getUuid()] = $item->getUuid();
        }

        foreach ($points as $place) {
            $parcelPoint = $this->shippingPointsFactory->create();
            try {
                $parcelPoint->load($place['point_uuid']);
                if (isset($oldPoints[$parcelPoint->getUuid()])) {
                    unset($oldPoints[$parcelPoint->getUuid()]);
                }
                // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
            } catch (Exception $e) {
                /*
                * silence , we don't care if this fails
                */
            }

            if (!$parcelPoint || !$parcelPoint->getUuid()) {
                $parcelPoint->setUuid($place['point_uuid']);
            }

            $parcelPoint->setShippingCode($place['point_shipping_code']);
            $parcelPoint->setName($place['point_name']);
            $parcelPoint->setRegion($place['point_region'] ?? '');
            $parcelPoint->setLocality($place['point_locality']);
            try {
                $parcelPoint->save();
                // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
            } catch (Exception $e) {
                /*
                * silence , we don't care if this fails
                */
            }
        }

        foreach ($oldPoints as $point) {
            $parcelPoint = $this->shippingPointsFactory->create();
            $parcelPoint->load($point);
            $parcelPoint->delete();
        }
    }
}
