<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Model\Observer;

use Exception;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\ShipmentCommentCreationInterfaceFactory;
use Magento\Sales\Api\Data\ShipmentItemCreationInterfaceFactory;
use Magento\Sales\Api\ShipOrderInterface;
use Magento\Sales\Model\Order;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use Psr\Log\LoggerInterface;

class AdminhtmlOrderSaveAfter implements ObserverInterface
{
    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var ShipOrderInterface
     */
    protected $shipOrder;

    /**
     * @var ShipmentItemCreationInterfaceFactory
     */
    protected $shipmentItemCreationFactory;

    /**
     * @var ShipmentCommentCreationInterfaceFactory
     */
    protected $shipmentCommentCreationInterfaceFactory;

    /**
     * @inheritDoc
     */
    public function __construct(
        LoggerInterface $logger,
        Config $config,
        ShipOrderInterface $shipOrder,
        ShipmentItemCreationInterfaceFactory $shipmentItemCreationInterfaceFactory,
        ShipmentCommentCreationInterfaceFactory $shipmentCommentCreationInterfaceFactory
    ) {
        $this->logger = $logger;
        $this->config = $config;
        $this->shipOrder = $shipOrder;
        $this->shipmentItemCreationFactory = $shipmentItemCreationInterfaceFactory;
        $this->shipmentCommentCreationInterfaceFactory = $shipmentCommentCreationInterfaceFactory;
    }

    /**
     * @inheritDoc
     */
    public function execute(Observer $observer)
    {
        $order = $observer->getEvent()->getOrder();

        if (!$order->getId()) {
            //order not saved in the database
            return $this;
        }
        $processingStatuses = [Order::STATE_PROCESSING, Order::STATE_PAYMENT_REVIEW];

        //if pending and create shipment
        $isActive = (boolean)$this->config->getValue('active') ?? false;
        $createShipment = $this->config->getValue('create_shipment_objects', $order->getStoreId());
        $createShipmentsOnProcessing = in_array($createShipment, $processingStatuses);
        $statusIsProcessing = in_array($order->getStatus(), $processingStatuses);

        if (!$order->getIsVirtual() &&
            $isActive === true &&
            $this->config->matchShippingMethod($order->getShippingMethod()) &&
            $createShipmentsOnProcessing &&
            $statusIsProcessing &&
            $order->getData('montonio_shipping_created') == '0' &&
            $order->canShip()
        ) {
            //@TODO we need a shipment comment
            $this->createShipment($order, __('Shipment created'));
        }

        return $this;
    }

    /**
     * Create shipment object
     *
     * @param OrderInterface $order
     * @param string|null $comment
     * @return void
     */
    protected function createShipment(
        OrderInterface $order,
        string $comment = null
    ): void {
        try {
            $items = [];
            foreach ($order->getItems() as $orderItem) {
                $shipmentItemCreation = $this->shipmentItemCreationFactory->create();
                $shipmentItemCreation->setOrderItemId($orderItem->getItemId());
                $shipmentItemCreation->setQty($orderItem->getQtyOrdered());
                $items[] = $shipmentItemCreation;
                $shipmentItemCreation = null;
            }

            if (!empty($comment)) {
                $shipmentComment = $this->shipmentCommentCreationInterfaceFactory->create();
                $shipmentComment->setComment(__($comment));
                $comment = $shipmentComment;
            }

            $this->shipOrder->execute(
                (int)$order->getEntityId(),
                $items,
                false,
                false,
                $comment,
                [],
                [],
                null
            );
            //unset the values
            $items = [];
        } catch (Exception $e) {
            $this->logger->info($e->getMessage());
        }
    }
}
